=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2012-01-28 00:41:30 +0000
+++ bzrlib/builtins.py	2012-01-31 15:43:17 +0000
@@ -6454,8 +6454,8 @@
     def run(self, location=None):
         if location is None:
             location = "."
-        branch = Branch.open_containing(location)[0]
-        branch.bzrdir.destroy_branch()
+        cdir = controldir.ControlDir.open_containing(location)[0]
+        cdir.destroy_branch()
 
 
 class cmd_shelve(Command):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2012-01-28 02:05:52 +0000
+++ bzrlib/bzrdir.py	2012-01-31 15:43:17 +0000
@@ -862,7 +862,7 @@
 
     def create_branch(self, name=None, repository=None,
             append_revisions_only=None):
-        """See BzrDir.create_branch."""
+        """See ControlDir.create_branch."""
         if name is None:
             name = self._get_selected_branch()
         return self._format.get_branch_format().initialize(self, name=name,
@@ -870,7 +870,7 @@
                 append_revisions_only=append_revisions_only)
 
     def destroy_branch(self, name=None):
-        """See BzrDir.create_branch."""
+        """See ControlDir.destroy_branch."""
         if name is None:
             name = self._get_selected_branch()
         path = self._get_branch_path(name)
@@ -885,7 +885,11 @@
                 self._write_branch_list(branches)
             finally:
                 self.control_files.unlock()
-        self.transport.delete_tree(path)
+        try:
+            self.transport.delete_tree(path)
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=urlutils.join(self.transport.base,
+                path), bzrdir=self)
 
     def create_repository(self, shared=False):
         """See BzrDir.create_repository."""

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2012-01-27 14:01:25 +0000
+++ bzrlib/controldir.py	2012-01-31 15:43:17 +0000
@@ -163,7 +163,7 @@
         """Create a branch in this ControlDir.
 
         :param name: Name of the colocated branch to create, None for
-            the default branch.
+            the user selected branch or "" for the active branch.
         :param append_revisions_only: Whether this branch should only allow
             appending new revisions to its history.
 
@@ -175,8 +175,9 @@
     def destroy_branch(self, name=None):
         """Destroy a branch in this ControlDir.
 
-        :param name: Name of the branch to destroy, None for the default 
-            branch.
+        :param name: Name of the branch to destroy, None for the 
+            user selected branch or "" for the active branch.
+        :raise NotBranchError: When the branch does not exist
         """
         raise NotImplementedError(self.destroy_branch)
 
@@ -302,7 +303,7 @@
     def _get_selected_branch(self):
         """Return the name of the branch selected by the user.
 
-        :return: Name of the branch selected by the user, or None.
+        :return: Name of the branch selected by the user, or "".
         """
         branch = self.root_transport.get_segment_parameters().get("branch")
         if branch is None:

=== modified file 'bzrlib/tests/blackbox/test_rmbranch.py'
--- bzrlib/tests/blackbox/test_rmbranch.py	2011-12-14 20:21:52 +0000
+++ bzrlib/tests/blackbox/test_rmbranch.py	2012-01-25 17:31:04 +0000
@@ -83,6 +83,6 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(5, self.hpss_calls)
+        self.assertLength(2, self.hpss_calls)
         self.assertLength(1, self.hpss_connections)
         self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2012-01-18 20:27:41 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2012-01-25 17:31:04 +0000
@@ -159,6 +159,14 @@
         bzrdir.create_branch()
         bzrdir.open_branch()
 
+    def test_destroy_branch_no_branch(self):
+        branch = self.make_repository('branch')
+        bzrdir = branch.bzrdir
+        try:
+            self.assertRaises(errors.NotBranchError, bzrdir.destroy_branch)
+        except (errors.UnsupportedOperation, errors.TransportNotPossible):
+            raise TestNotApplicable('Format does not support destroying branch')
+
     def test_destroy_repository(self):
         repo = self.make_repository('repository')
         bzrdir = repo.bzrdir

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2012-01-28 00:41:30 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-01-31 15:43:17 +0000
@@ -86,6 +86,9 @@
 * ``ControlDir`` now has a new method ``set_branch_reference`` which can
   be used for setting branch references. (Jelmer Vernooij)
 
+* ``ControlDir.destroy_branch`` now raises ``NotBranchError`` rather than
+  ``NoSuchFile`` if the branch didn't exist. (Jelmer Vernooij, #921693)
+
 Internals
 *********
 

